/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
 
 import zero.core.etags.Etag
 import zero.ide.services.AppUtils
 import zero.ide.services.FileUtils
 import zero.ide.services.ExtensionFileFilter
 import zero.ide.services.HiddenFileFilter
 import zero.ide.services.TemplateGenerator
 import zero.cli.tasks.utils.ToolsUtils
 import zero.core.utils.VDirectory
 import zero.appbuilder.resources.MessageHandler
 import zero.core.utils.ClientMessageFormatter
 import zero.core.context.GlobalContext;
 import zero.core.context.GlobalContextURIs;

 //
// getETag() returns the ETag for a file as an unquoted string.
 //
 def getETag(target)
 {
  def modified = target.lastModified().toString();
  return Etag.computeHashedValue(modified);
 }
    
 def onUpdate()
 {
	def componentType = "iwidget";
	def contextRoot=GlobalContext.zget('/config/contextRoot');
	if (contextRoot==null) {
		contextRoot='';
 	}
	
	def uriPrefix = contextRoot + '/resources/applications';
	def requestUri = request.uri[];
 	if (!requestUri.startsWith(uriPrefix)) {
 		// onUpdate is currently only valid within the application
 		request.status = 405
 		return
 	}   
 	
 	def ifMatch = zget("/request/headers/in/If-Match");
 	def ifNoneMatch = zget("/request/headers/in/If-None-Match");

 	//
 	// remove quotes from etag value
 	//
 	if (ifMatch != null)
 		ifMatch = ifMatch[1..-2];

 	def isCreating = ifNoneMatch != null && ifNoneMatch.equals("*");    
 	
 	if (isCreating)
 	{
 		//
 		// can't be creating a new file and matching against an existing
 		// version of a file at the same time
 		//
 		if (ifMatch != null)
 		{
 			request.status = 412;
 			return;
 		}

 		handleCreate(componentType);
 	}
 	else
 	{
 		// Can't do an update through this method
 		request.status = 412;
 		return;
 	}
 }

 def fullyQualify(fileName)
 {
     def apphome = AppUtils.getApphome(request.params.applicationsId[])
     File f = new File(apphome, fileName)
     return f
 }

 def dprintln(String value)
 {
 	System.out.println(value)
 }

 def dumpInfo(componentType)
 {
 	def appId = request.params.applicationsId[]
 	dprintln("appId="+appId)
 	def compId = zget("/request/params/"+componentType+"/Id")
 	dprintln("compId="+compId)
 	def templateName = request.params.template[0]
 	if (templateName)
 		dprintln("template="+templateName)
 	def iwdesc = request.params.iwdesc[0]
 	dprintln("iwdesc="+iwdesc);
 	def iwname = request.params.iwname[0]
 	dprintln("iwnam="+iwname);

 }
  

 /**
  * @success 200 returns the component metadata { commandRunning: [true|false] }
  * @error 404 if the
  */  
 def handleCreate(componentType)
 { 	    
    // The first directory in the path comes in as the ID    
    //dumpInfo(componentType)
	def pathName1 = zget("/request/params/"+componentType+"Id")
	def pathName2 = request.params.iwname[0]
    if (!pathName1 || !pathName2) {
 		request.status = 404 
 		return
    }    	
    	
    def relativeTargetName = "app/iwidgets/"+pathName1 // +pathName2
    this.resultUri = "resources/applications/"+request.params.applicationsId[]+"/files/"+relativeTargetName+"/"+request.params.iwname[0]+".xml"
    
    def targetFile = fullyQualify(relativeTargetName)		
    if (targetFile.exists())
    {
 		// File already exists!
 	    request.status = 412;
		request.view = 'error'
		request.headers.out.'Content-Type' = 'application/json'
		ClientMessageFormatter messageFormatter = new ClientMessageFormatter("zero/appbuilder/core/resources/consolemessages");
		request.error.message = messageFormatter.getMessage('FILE_ALREADY_EXISTS', targetFile.absolutePath);
		render() 		
 	    return;
    }

    def templateName = request.params.template[0]
    
    if ((!templateName) || (templateName.length() == 0))
    {	   
       // template required
 	   request.status = 404;
 	   return;
    }
    else
    {
        
        def iwusecreatetestfile = request.params.iwusecreatetestfile[0]
        if ((iwusecreatetestfile == "on") || (iwusecreatetestfile == "true")) {
    	   def iwcreatetestfileinput = request.params.iwcreatetestfileinput[0]
    	   def targetFile2 = new File(targetFile.getParentFile().getParentFile().getParentFile(),"/public/"+iwcreatetestfileinput);
    	   if (targetFile2.exists())
    	    {
    	 		// File already exists!
    	 	    request.status = 412;
    			request.view = 'error'
    			request.headers.out.'Content-Type' = 'application/json'
    			ClientMessageFormatter messageFormatter = new ClientMessageFormatter("zero/appbuilder/core/resources/consolemessages");
    			request.error.message = messageFormatter.getMessage('FILE_ALREADY_EXISTS', targetFile2.absolutePath);
    			render() 		
    	 	    return;
    	    }      
        }

    	
    	def templateDir = VDirectory.getVFile("/app/templates/"+componentType+"/"+templateName)  
    	if ((templateDir == null) || (!templateDir.exists()) || (!templateDir.isDirectory()))
    	{
    		request.status = 404
    		return
    	}		    
   		generateDirectory(targetFile, relativeTargetName, templateDir, templateName)
    
      response = targetFile
      request.headers.out.Location = resultUri
      request.headers.out.ETag = "\"${getETag(targetFile)}\"".toString();
      request.view = 'JSON'
      request.json.output = [ 'url' : resultUri ]
      request.status = 200
      render()   
    }
 }
  
 def copyFile(templateFile, targetFile) 
 {
 	ToolsUtils.copyFile(templateFile, targetFile)
 } 

 
 def generateDirectory(targetDir, relativeTargetName, templateDir, templateName) 
 {
 	 TemplateGenerator tg = new TemplateGenerator(templateDir, targetDir)
 	  	 
 	 tg.setContextVariable(TemplateGenerator.templateNameVar, templateName)
 	 
 	 def appName = zget("/storage/applications#"+request.params.applicationsId[]+"/name") 	 
 	 tg.setContextVariable(TemplateGenerator.appNameVar,appName)
 	 

 	 def iwdesc = request.params.iwdesc[0]
 	 tg.setContextVariable("iwDescription",iwdesc)
 	  	 
 	 def iwname = request.params.iwname[0]
 	 tg.setContextVariable(TemplateGenerator.nameVar,iwname)
 	 tg.setContextVariable("iwName",iwname)
 
 	 
     def iwusecreatetestfile = request.params.iwusecreatetestfile[0]
     //System.out.println("in iwidget.groovy iwusecreatetestfile ="+iwusecreatetestfile)
     if ((iwusecreatetestfile == "on") || (iwusecreatetestfile == "true")) {
	   tg.setContextVariable("iwUseCreateTestFile",iwusecreatetestfile) 	  		 
       //System.out.println("in iwidget.groovy iwUseCreateTestFile set to  ="+iwusecreatetestfile)
 	   def iwcreatetestfileinput = request.params.iwcreatetestfileinput[0]
       //System.out.println("in iwidget.groovy iwCreateTestFileInput set to  ="+iwcreatetestfileinput)
 	   tg.setContextVariable("iwCreateTestFileInput",iwcreatetestfileinput)
       def iwurl = "/iwidgets/"+ iwname +"/"+iwname+".xml"
 	   tg.setContextVariable("iwURL",iwurl) 
 	   // NEED to deal with iwCreateFull
 	   
       def testTempName = "iwidgettest/"+"Basic iWidget Test" +"/default.html"
       if (templateName == "Sample iWidget") {
    	   testTempName = "iwidgettest/"+"Sample iWidget Test" +"/default.html"
       }
 	   def testTemplate = new File(templateDir.getParentFile().getParentFile(),testTempName) 
 	   def testTarget = new File(targetDir.getParentFile().getParentFile().getParentFile(),"/public/"+iwcreatetestfileinput) 
 	   tg.generate(testTemplate, testTarget);
     }
 	 
 	 
 	 
 	 // Invoke the generator, telling it to replace all occurrences of the template name with 
 	 tg.generate()
 }
 
 def createWithoutTemplate(targetFile)
 {	
     targetFile.getParentFile().mkdirs()
     
     def output = new FileOutputStream(targetFile)
     
     output.flush()
     output.close()        
 }

