/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */

 
 import zero.core.etags.Etag
 import zero.ide.services.AppUtils
 import zero.ide.services.FileUtils
 import zero.ide.services.ExtensionFileFilter
 import zero.ide.services.HiddenFileFilter
 import zero.ide.services.TemplateGenerator
 import zero.cli.tasks.utils.ToolsUtils
 import zero.core.utils.VDirectory
 import zero.appbuilder.resources.MessageHandler
 import zero.core.utils.ClientMessageFormatter
 import zero.core.context.GlobalContext;
 import zero.core.context.GlobalContextURIs;
 import zero.json.Json;
 import zero.util.UTF8FileReader;
 import zero.util.UTF8FileWriter;
 
 class statics {
	    public static APP_FORMS_DIR = "public/zero/forms/";
	    public static APP_MODELS_DIR = "app/models/";
	    public static   FORM_MODEL_TYPES = [
	        "Text":"string",
	        "Time":"time",
	        "Date":"date",
	        "Checkbox":"boolean",
	        "Select":"string"
	    ];
	    public static   MODEL_FORM_TYPES = [
	        "string":"Text",
	        "decimal":"Number",
	        "integer":"Number",
	        "float":"Number",
	        "boolean":"Checkbox",
	        "date":"Date",
	        "time":"Time",
	        "date-time":"Time"
	    ];
	}
 
def onUpdate() 
{
	//Get the data, and generate the artifacts according to the wizard
	def componentType = "iwidgettest";
	def contextRoot=GlobalContext.zget('/config/contextRoot');
	if (contextRoot==null) {
		contextRoot='';
 	}
	
	def uriPrefix = contextRoot + '/resources/applications';
	def requestUri = request.uri[];
 	if (!requestUri.startsWith(uriPrefix)) {
 		// onUpdate is currently only valid within the application
 		request.status = 405
 		return
 	}   
 	
 	def appId = request.params.applicationsId[]
 	def zrmId = request.params.zrmId[]
	
	def data = request.params.data[]
	
	data = Json.decode(data);
 	
 	 def apphome = AppUtils.getApphome(appId);
     def fullName = "/app/models/" + zrmId +".json"
     File targetFile = new File(apphome, fullName);
	 if (targetFile.exists())
    {
 		// File already exists!
 	    request.status = 412;
		request.view = 'error'
		request.headers.out.'Content-Type' = 'application/json'
		ClientMessageFormatter messageFormatter = new ClientMessageFormatter("zero/appbuilder/core/resources/consolemessages");
		request.error.message = messageFormatter.getMessage('FILE_ALREADY_EXISTS', targetFile.absolutePath);
		render() 		
 	    return;
    }
	 
	 //	Generate ZRM model file
	generateZRMModel(data.model.name,data.model.template,data.model.isBasedForm,data.model.basedFormName)
 	
 	//Generate handler
	if(data.handler.isCreateHandler){
		generateHandler(zrmId,data.handler.name,data.handler.isGenerateBnd,data.handler.nestedParentResource)
	}
 	//Generate form
	if(data.form.isGenerateForm){
		generateZeroForm(zrmId,data.form.formName)
	}
 	
 	//Generate Test HTML 
	if(data.test.isGenerateTest){
		generateTestHTML(data.model.name,data.test.fileName)
	}
 	fullName = "/app/models/" + zrmId +".json"
	def resultUri = "resources/applications/"+request.params.applicationsId[]+"/files/"+fullName;  
	request.headers.out.Location = resultUri
	request.view = 'JSON'
	request.json.output = [ 'url' : resultUri ]
	request.status = 200
	render()   
}


def generateZRMModel(name,templateName,isBasedForm,basedFormName){
	if(!isBasedForm){
		 def apphome = AppUtils.getApphome(request.params.applicationsId[]);
		   def fullName = "/app/models/" + name +".json"
		   File targetFile = new File(apphome, fullName);
		   
		   if ((!templateName) || (templateName.equalsIgnoreCase("* None *")))
		   {	   
			   invokeMethod("FileTemplateHandler.groovy", "createWithoutTemplate", targetFile)
		   }else{
			   def templateFile = VDirectory.getVFile("/app/templates/models/"+templateName)  
			   targetFile = invokeMethod("FileTemplateHandler.groovy", "generateFile", targetFile,templateFile,templateName)	
		   }
	}else{
		
		def targets = getFormModelTargets(name,basedFormName);    
	    if (targets.form == null || !targets.form.exists())
	    {   // form does not exist
	        request.status = 404
	        return
	    } 
	    def jsons = formToModel(targets.form, targets.model.name.split("\\.")[0]);
	    if (jsons != null && writeJSONtoFile(targets.model, jsons.model))
	    {
	    	if (jsons.form != null)
	    	{
	    		//Why need write something to form???
	    		writeJSONtoFile(targets.form, jsons.form);
	    	}
	    }
	}
	
}

def getFormModelTargets(resourceName,formName)
{
	def apphome = AppUtils.getApphome(request.params.applicationsId[])    
    if (!apphome){
        return null
    }
	 def resourceFullName = "/app/models/" + resourceName +".json"
	 def formFullName = "/public/zero/forms/" +formName
    
    def formFile = new File(apphome, formFullName);
	 def restURI = "/resources/applications/"+request.params.applicationsId[]+"/files"+formFullName; 
    def modelFile =  new File(apphome, resourceFullName);
    return [form:formFile, model:modelFile, uri: restURI];
}

def writeJSONtoFile(pTarget, pJsonObject)
{
    if (pJsonObject == null || pTarget.isDirectory())
    {
        pTarget.createNewFile();
        return false;
    }
    
    def parentFolder = pTarget.getParentFile();
    if (parentFolder != null)
    	parentFolder.mkdirs();
    
    def output = new UTF8FileWriter(pTarget);
    zero.json.Json.encode(pJsonObject, output, true);
    return true;
}

def formToModel(pFormFile, pModelName)
{
    
    def modelMap = [:];
    modelMap.fields = [:];
    def formMap;
    def formModified = false;
    try {
        // create a map representation of the JSON form     
        formMap = zero.json.Json.decode(new UTF8FileReader(pFormFile), true);
       

        // add model name to form - if absent
         if (formMap.model == null || formMap.model.equals("")) {
            formMap.model = pModelName
            formModified = true;
         }
        	
        // iterate over fields
        def modelField;
        formMap.fields.each { fieldAttMap ->
            modelField = [:];
            modelField.label = (fieldAttMap.title != null ? fieldAttMap.title : fieldAttMap.dataId);
            modelField.required =  (fieldAttMap.required != null ?  fieldAttMap.required : false);  
            modelField.type = getModelType(fieldAttMap);  
            modelField.description =  (fieldAttMap.description  != null  ? fieldAttMap.description : ""); 
            modelField.default_value =  ""+(fieldAttMap.default_value  != null  ? fieldAttMap.default_value : "");  
            setAdditionalAttributes(fieldAttMap, modelField);
            // add data-id to form-field, if absent
            if (fieldAttMap.dataId == null || fieldAttMap.dataId == "") {
            	fieldAttMap.dataId = fieldAttMap.shortName;
            	formModified = true;
            }
            modelMap.fields."${fieldAttMap.dataId}" = modelField;
        }
    } catch (Exception e)
    {
    	setConflict("Error reading Zero Resource Form source ${pFormFile}: ${e.class.name} [${e.message}]");
        return null;
    }
    if (formModified)
    	return [model:modelMap, form:formMap];
    else
    	return [model:modelMap];
}

//
//setConflict() sets a conflict while reading a JSON model source
//
def setConflict(pMessage)
{
	request.status = 409;
	logger.SEVERE ({ pMessage });
}

//
//getETag() returns the ETag for a file as an unquoted string.
//
def getETag(pTarget)
{
def modified = pTarget.lastModified().toString();
return Etag.computeHashedValue(modified);
}

// Helper methods for tranforming a form model to a ZRM model
def getModelType(pFieldAttMap)
{
	if (pFieldAttMap.type == 'Number')
		return pFieldAttMap.numberType;
	else
		return statics.FORM_MODEL_TYPES[pFieldAttMap.type]; 
}

def setAdditionalAttributes(pFieldAttMap, pModelField)
{
	if (pModelField.type == 'decimal')	
	{
		if (pFieldAttMap.digitPlaces != null) {
			pModelField.decimal_places = pFieldAttMap.digitPlaces;
			if (pFieldAttMap.min != null && pFieldAttMap.max != null)
				// Set max-digits based on the biggest of min and max, plus decimal places
				pModelField.max_digits = ("" + Math.max(Math.abs(pFieldAttMap.min), Math.abs(pFieldAttMap.max))).length() + pModelField.decimal_places;
			else
				pModelField.max_digits = 10;
		}
	}

    if(pFieldAttMap.options)
    {
        pModelField.options = [];
        pFieldAttMap.options.each { option ->
               pModelField.options.add(option.val);
               if (option.opt.equals(pFieldAttMap.selected_option))
                   pModelField.default_value = option.val;
        }
    }
}


def generateHandler(resourceName,handlerName,isGeneratedBnd,nestedParentResource){
	 def apphome = AppUtils.getApphome(request.params.applicationsId[]);
	   def fullName = "/app/resources/" + handlerName
	   File targetFile = new File(apphome, fullName);
	   def templateName = "zrm.groovy"
	   if(fullName.endsWith(".php")){
		   templateName = "zrm.php"
	   }
	   def templateFile = VDirectory.getVFile("/app/templates/resourcehandlers/"+templateName)  
	   targetFile = invokeMethod("FileTemplateHandler.groovy", "generateFile", targetFile,templateFile,templateName)	
	   
	   if(isGeneratedBnd){
			 def dndFullName = "/app/resources/" + resourceName + ".bnd"
			 File dndTargetFile = new File(apphome, dndFullName);
			 if(nestedParentResource.endsWith(".json")){
				 nestedParentResource = nestedParentResource.replace(".json","")
			 }
			 def dndString = nestedParentResource + "/" + resourceName
			def output = new UTF8FileWriter(dndTargetFile);
			 output.write(dndString,0,dndString.length())
			 output.close()
	   }
}


//Whether the form is generated based the ZRM model
//Simple for an empty firstly
def generateZeroForm(resourceName,formName){
	def targets = getModelFormTargets(resourceName,formName);    
    
    if (targets.model == null || !targets.model.exists())
    {   // model does not exist
        request.status = 404
        return
    } 
    def form = modelToForm(targets.model);
    writeJSONtoFile(targets.form, form)
}

def getModelFormTargets(resourceName,formName)
{
	def apphome = AppUtils.getApphome(request.params.applicationsId[])    
    if (!apphome){
        return null
    }
	 def resourceFullName = "/app/models/" + resourceName +".json"
	 def formFullName = "/public/zero/forms/" +formName
    
    def formFile = new File(apphome, formFullName);
	 def restURI = "/resources/applications/"+request.params.applicationsId[]+"/files"+resourceFullName; 
    def modelFile =  new File(apphome, resourceFullName);
    return [model:modelFile, form:formFile, uri: restURI];
}

def modelToForm(pModelFile)
{
    
    def formMap = [:];
    
    try {
        // create a map representation of the JSON model     
        def modelMap = zero.json.Json.decode(new UTF8FileReader(pModelFile), true);
        // set form model
        formMap.model = pModelFile.name.split("\\.")[0];
        formMap.title = "Title";
        formMap.description = "Description";
        formMap.fields = [];        
        // iterate over fileds
        def formField;
        modelMap.fields.each { fieldId, fieldAttMap ->
            formField = [:];
            formField.dataId = fieldId;
            formField.title =  (!isEmpty(fieldAttMap.label) ? fieldAttMap.label : fieldId);
            formField.shortName = (fieldId.endsWith("_id") ? fieldId :  fieldId+ "_id");
            formField.required =  (fieldAttMap.required != null ? fieldAttMap.required : false);  
            formField.type = getFormType(fieldAttMap);  
            formField.description =  (!isEmpty(fieldAttMap.description) ? fieldAttMap.description : ""); 
            formField.default_value =  (!isEmpty(fieldAttMap.default_value) ? fieldAttMap.default_value : "");  
            formField."immutable" = false;
            setFormAdditionalAttributes(fieldAttMap, formField);
            formField.onMouseOut = "";
            formField.onChange = "";
            formField.onMouseOver = "";
            formField.onClick = "";
            formMap.fields.add(formField);
        }
    } catch (Exception e)
    {
    	setConflict("Error reading Zero Resource Model source ${pModelFile}: ${e.class.name} [${e.message}]");
        return null;
    }
    return formMap;
    
}


// Helper method for checking whether a field is empty
def isEmpty(pFieldValue)
{
	return (pFieldValue == null || pFieldValue.trim().equals(""));
}

// Helper methods for tranforming a ZRM model to a form model
def getFormType(pFieldAttMap)
{
    return (pFieldAttMap.options) ? "Select" : statics.MODEL_FORM_TYPES[pFieldAttMap.type];
    
}

def setFormAdditionalAttributes(pFieldAttMap, pFormField)
{
    if(pFieldAttMap.type.equals('integer'))
    {
        pFormField.numberType = 'integer';
        pFormField.digitPlaces = 0;
    }

    if(pFieldAttMap.type.equals('decimal'))
    {
        pFormField.numberType = 'decimal';
        pFormField.digitPlaces = pFieldAttMap.decimal_places;
        def minMax = Double.parseDouble(getNinesString(pFieldAttMap.max_digits-pFieldAttMap.decimal_places));
        pFormField.max = minMax;
        pFormField.min = -minMax;
        minMax = "0." + getNinesString(pFieldAttMap.decimal_places);
        minMax = Double.parseDouble(minMax);
        pFormField.max = pFormField.max + minMax;
        pFormField.min = pFormField.min - minMax;
    }
    
    if(pFieldAttMap.options)
    {
        pFormField.options = [];
        pFieldAttMap.options.each { optionVal ->
               pFormField.options.add([opt:optionVal,val:optionVal]);
        }
        pFormField.selected_option = (pFieldAttMap.default_value != null ? pFieldAttMap.default_value : "");
    }
    else
    {
        if(!pFieldAttMap.type.equals('boolean'))
        {
        	pFormField.onValid = "";
        	pFormField.onInvalid = "";
        }   	
    }
}

def getNinesString(pNumberOfNines)
{
        if(!pNumberOfNines) {
            return "0";
        }
        def str = '9';
        for (int j=1; j<pNumberOfNines; j++)
        {
            str  = str + '9';
        }
        return str;
}



//
def generateTestHTML(resourceName,testFileName){
	 def apphome = AppUtils.getApphome(request.params.applicationsId[]);
	 def fullName = "/public/" + testFileName
	 File targetFile = new File(apphome, fullName);
	 def templateFile = VDirectory.getVFile("/app/templates/zrmtest/default.html")  
	 
	 TemplateGenerator tg = new TemplateGenerator()
	 tg.setContextVariable("resourceName",resourceName)
	 tg.generate(templateFile, targetFile)
	
}
