/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */


// 
// File: /app/scripts/new.groovy
// 
// Author: Brett King (brettk)
// 
// This script includes the API for the IDE's New File Menu
//

/** 
 *
 * @success 200 Successfully retrieved New File Menu information  
 * @error 500 If there was a configuration error
 * @format application/json
 * @example
 * To request the new file menu information, GET /appbuilder/menus/new
 *
 * Result:  JSON result that looks like this:
 * {
 *  "newfilehandlers": [
 *     {
 *     	  "componentType" : "css"     
 *        "extensions": [
 *           ".css",
 *           ".css.tmpl"
 *        ],
 *        "path": "/public/",
 *        "newMenuLabel": "Style Sheet in /public",
 *        "example": "/public/new.css",
 *        "dialogWidget": "zero.ide.widget.appedit.FileOpeningDialog.js",
 *        "iconClass": "newFileIconFiles"
 *     },
 *     {
 *     	  "componentType" : "flow"     
 *        "extensions": [
 *           ".flow",
 *           ".flow.tmpl"
 *        ],
 *        "path": "/public/",
 *        "newMenuLabel": "Flow in /public",
 *        "example": "/public/new.flow",
 *        "dialogWidget": "zero.ide.widget.appedit.FileOpeningDialog.js",
 *        "iconClass": "newFileIconFiles"
 *     },
 *     {
 *     	  "componentType" : "html"
 *        "extensions": [
 *           ".html",
 *           ".htm",
 *           ".html.tmpl",
 *           ".htm.tmpl"
 *        ],
 *        "path": "/public/",
 *        "newMenuLabel": "HTML Page in /public",
 *        "example": "/public/index.html",
 *        "dialogWidget": "zero.ide.widget.appedit.FileOpeningDialog.js",
 *        "iconClass": "newFileIconFiles"
 *     }
 *   }
 *
 */
def onGET()
{	
	def compList = zlist("/config/components")
	def handlers = []
	compList.each {
		def componentType = it.substring(it.lastIndexOf('/')+1)
			
		def compConfig = zget("/config/components/"+componentType)
		compConfig.put("componentType", componentType);
		
		handlers << compConfig
	}

	def result = [ 'newfilehandlers': handlers ]
			
	request.view = 'JSON'
	request.json.output = result
	render()
}