/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */


// 
// File: /app/scripts/FileTemplateHandler.groovy
// 
// Author: Brett King (brettk)
// 
// This script includes the RESTful API for the IDE's template-based component
// creation system
//

import zero.ide.services.ExtensionFileFilter
import zero.ide.services.HiddenFileFilter
import zero.core.utils.VDirectory

def createChildren(componentType, filter, templateRequired)
{	
    def children = VDirectory.listFiles("/app/templates/${componentType}", filter)
    
    def items = []
    
    children.each() {
        items << [name:it.getName(),
                  description:getDescription(componentType, it.getName()),
                  icon:getIconUrl(componentType, it.getName())]
    }
    if (!templateRequired) {
    	items << [name:"* None *",
    		  description:"Create an empty file"
    		  //icon:getIconUrl("/app/templates/default.png")
    		 ]
    }
    return items;
}

def getDescription(componentType, templateName)
{	
	def descFile = VDirectory.getVFile("/app/templates/${componentType}/${templateName}.desc")	
	def description	
	if (descFile != null)
	{
		description = descFile.text
	}	
	else
	{
		description = templateName
	}
	return description
}

def getIconUrl(componentType, templateName)
{	
	def iconFile = VDirectory.getVFile("/app/templates/${componentType}/${templateName}.png")	
	def icon
	if (iconFile != null)
	{
		icon = iconFile.getName()
	}	
	else
	{
		icon = ''
	}
	return icon
}

def onGET()
{	
	def componentType = "${event.type[]}";	
	def compConfig = zget("/config/components/"+componentType)
	def extensions = compConfig.get("extensions")
	def templateRequired = compConfig.get("templateRequired")
	
	def filter = new HiddenFileFilter(extensions, true);
	def templates = createChildren(componentType, filter, templateRequired)
	
	if ((templates) && (templates.size() >0)) {
		def result = [ 'identifier': 'name',
		               'items': templates ]				
		request.view = 'JSON'
		request.json.output = result
		render()
	}
	else
	{
		request.status = 404;
	}
}